from django.contrib.admin import ACTION_CHECKBOX_NAME
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse

from gaby.contacts.models import (
    Adhesion,
    DonneesBio,
    Personne,
    ProjetBio,
    Rdv,
    Structure,
)


class TestAdminAction(TestCase):
    """
    fat test pour verifier les csv, j'utilise unittest qui un peu plus
    leger lorsqu'on utilise massivement des fixtures en base de donnée
    """

    # le test csv verifie: status_code == 200 et 'text/csv' in content-type
    # CSV_COMPARE verifie en plus le contenu du csv généré
    CSV_COMPARE = True

    fixtures = [
        'demo/01_groups',
        'demo/02_users',
        'demo/10_ref-light',
        'demo/20_structures',
        'demo/21_personnes',
        'demo/225_rdv',
        'demo/226_rdv_personne.json',
        'demo/22_projetbio',
        'demo/23_adhesiongab',
        'demo/24_donneesbio',
        'demo/25_assolement',
        'demo/26_commercialisation',
        'demo/27_transformation',
        'demo/28_activitestructure',
    ]

    def setUp(self):
        "don't bother with login"
        admin = User.objects.get(username='admin')
        self.client.force_login(admin)

    def _test_call_admin_action(self, model, action):
        "call admin action 'action' on whole 'model' list "
        app_label = model._meta.app_label
        model_name = model._meta.model_name
        change_url = reverse(
            'admin:{0}_{1}_changelist'.format(app_label, model_name)
        )
        pk_list = model.objects.values_list('pk', flat=True)
        data = {'action': action, ACTION_CHECKBOX_NAME: pk_list}
        ret_post = self.client.post(change_url, data, follow=True)
        assert ret_post.status_code == 200
        return ret_post

    def _test_gen_csv(self, model, action):
        ret_post = self._test_call_admin_action(model, action)
        model_name = model._meta.model_name
        csv_filename = '{0}_{1}.csv'.format(model_name, action)
        path_test = 'gaby/contacts/tests/'
        path_tmp = '/tmp/'
        assert 'text/csv' in ret_post.get('content-type')
        with open(path_tmp + csv_filename, 'w+b') as csv_file:
            csv_file.write(ret_post.content)
        if not self.CSV_COMPARE:
            return
        with open(path_test + csv_filename, 'r+b') as csv_check_file:
            self.assertEqual(
                csv_check_file.read(),
                ret_post.content,
                "\ncheck following with a spreadsheat and/or"
                " meld\n{0} {1}".format(
                    path_tmp + csv_filename, path_test + csv_filename
                ),
            )

    def _test_gen_mailto(self, model, action):
        ret_post = self._test_call_admin_action(model, action)
        expected_breadcrumbs = (
            '<div class="breadcrumbs">'
            '<a href="/private/">Accueil</a> &rsaquo; Envoi de courriel'
            '</div>'
        )
        self.assertContains(ret_post, expected_breadcrumbs, html=True)

    def test_structure_gen_csv_complet(self):
        self._test_gen_csv(Structure, 'gen_csv_complet')

    def test_structure_gen_csv_publipostage_pphy(self):
        self._test_gen_csv(Structure, 'gen_csv_publipostage_pphy')

    def test_structure_gen_csv_publipostage_wo_pphy(self):
        self._test_gen_csv(Structure, 'gen_csv_publipostage_wo_pphy')

    def test_personne_gen_csv_complet(self):
        self._test_gen_csv(Personne, 'gen_csv_complet')

    def test_personne_gen_csv_publipostage(self):
        self._test_gen_csv(Personne, 'gen_csv_publipostage')

    def test_projetbio_gen_csv_complet(self):
        self._test_gen_csv(ProjetBio, 'gen_csv_complet')

    def test_donneesbio_gen_csv_complet(self):
        self._test_gen_csv(DonneesBio, 'gen_csv_complet')

    def test_donneesbio_gen_csv_assolement(self):
        self._test_gen_csv(DonneesBio, 'gen_csv_assolement')

    def test_donneesbio_gen_csv_commercialisation(self):
        self._test_gen_csv(DonneesBio, 'gen_csv_commercialisation')

    def test_donneesbio_gen_csv_transformation(self):
        self._test_gen_csv(DonneesBio, 'gen_csv_transformation')

    def test_adhesion_gen_csv_complet(self):
        self._test_gen_csv(Adhesion, 'gen_csv_complet')

    def test_rdv_gen_csv_rdv_ind(self):
        self._test_gen_csv(Rdv, 'gen_csv_rdv_ind')

    def test_rdv_gen_csv_rdv_coll(self):
        self._test_gen_csv(Rdv, 'gen_csv_rdv_coll')

    def test_rdv_gen_csv_rdv_coll_det(self):
        self._test_gen_csv(Rdv, 'gen_csv_rdv_coll_det')

    def test_personne_gen_mailto(self):
        self._test_gen_mailto(Personne, 'gen_mailto')

    def test_structure_gen_mailto(self):
        self._test_gen_mailto(Structure, 'gen_mailto')

    def test_structure_gen_mailto_sympa(self):
        self._test_gen_mailto(Structure, 'gen_mailto_sympa')
