from django.core.exceptions import ValidationError

import pytest

from ..models import AutreTerritoire, Commune, InterCommunalite


class TestStr:
    def test_intercommunalite(self):
        instance = InterCommunalite(nom='dourges')
        assert str(instance) == 'dourges'

    def test_autreterritoire(self):
        instance = AutreTerritoire(nom='neverland')
        assert str(instance) == 'neverland'

    def test_commune(self):
        instance = Commune(nom='lievin', code_postal='62800')
        assert instance.code_et_ville() == '62800 LIEVIN'


class TestCommuneCodePostal:
    @pytest.mark.parametrize(
        ('code_postal', 'code_insee', 'exp_res', 'exp_fail'),
        [
            ('', None, '', False),  # vide
            ('   ', None, '', False),  # vide
            ('62300', None, '62300', False),  # cp simple
            (' 62300 ', None, '62300', False),  # cp simple a nettoyer
            ('B-4110', None, 'B-4110', False),  # cp belge
            ('b-4110', None, 'B-4110', False),  # cp belge
            ('b-4110', 411, 'B-4110', True),  # cp belge avec un code insee
            ('2345', None, '2345', False),  # cp
            ('2345', 123, '02345', False),  # cp fr a padder
            ('2A345', 123, '2A345', False),  # cp fr corse
            ('2B345', 123, '2B345', False),  # cp fr corse
            ('2C345', 123, '2C345', True),  # cp fr invalid
            ('invalid', None, '', True),  # cp invalid
            ('invalid', 0, '', True),  # cp fr invalid
        ],
    )
    def test_clean_code_postal(
        self, code_postal, code_insee, exp_res, exp_fail
    ):
        instance = Commune(code_postal=code_postal, code_insee=code_insee)
        try:
            instance._clean_code_postal()
        except ValidationError:
            has_failed = True
        else:
            has_failed = False
        assert has_failed == exp_fail
        if not exp_fail:
            assert instance.code_postal == exp_res

    def test_commune_clean(self):
        instance = Commune()
        instance.clean()
        assert instance.code_postal == ''
